function [results]=multiple_dsets_cauchy(delta_acc,how_many_dsets,n_runs,reps)
%[results]=multiple_dsets_cauchy(delta_acc,how_many_dsets,nruns,reps)
%
%Analyzes multiple data sets through the Poisson and Wilcoxon 
%signed rank test.
%Zhe difference of accuracy between two classifiers is generate on each data set from a 
%Student distribution with mean=delta_acc, scale parameter=delta_acc and one degree of
%freedom. This is a Cauchy distribution. The sample differences are capped to +0.5 or -0.5, in case they are larger than 0.5 or smaller than -0.5.
%
%-how_many_dsets: how many data sets used for inference on multiple data
%sets
%
%-n_runs: how many runs of 10-folds-cross-validation are performed on each data set.
%Notice that we assume 10-folds as a default for cross-validation.
%
%-reps:how many repetitions of the experiment
%
%==saves the results both to csv and to mat file.
%
%==Example (100 repetitions under the null hypothesis, 50 dsets and 10 runs
%of 10-folds cross-validation)
%[results]=multiple_dsets_cauchy(0,50,10,100)

nfolds=10;
sample_sizes=[25 50 100 250 500 1000];
%set the seed for reproducible results
randn('seed',0);


if delta_acc>=0.5
    error('delta_acc should be <= 0.5');
end

if delta_acc<0
    error('delta_acc should be > 0');
end

counter=1;
for j=1:length(delta_acc)
    for k=1:length(how_many_dsets)
        settings(counter).delta_acc=delta_acc(j);
        settings(counter).dsets=how_many_dsets(k);
        counter=counter+1;
    end
end




%helping variables
if delta_acc==0
    cauchy_dist = makedist('tLocationScale','mu',delta_acc,'sigma',0.010,'nu',1);
else    
    cauchy_dist = makedist('tLocationScale','mu',delta_acc,'sigma',delta_acc,'nu',1);
end
rho=1/nfolds;
ttest_poisson_h1=ones(reps,1);
sign_rank_p_value=ttest_poisson_h1;
student_p_value=ttest_poisson_h1;
results(length(settings)).sample_size=[];



%simulation
for j=1:length(settings)
    disp('setting');
    disp(j);
    current_many_dsets = settings(j).dsets;
    current_delta_acc=settings(j).delta_acc;
    diff_acc_a_b_each_dset=ones(current_many_dsets,1);
    dset_sizes=ones(length(how_many_dsets),1);  
    ttest_prob_h1=ones(current_many_dsets,1);
    
    for k=1:reps
        if mod(k,100)==0;
            disp('reps');
            disp(k);
        end
        
        Diff_ab=zeros(n_runs*nfolds,current_many_dsets);
        delta_acc_each_dset = random(cauchy_dist,current_many_dsets,1);
        delta_acc_each_dset(delta_acc_each_dset>0.5)=0.5;
        delta_acc_each_dset(delta_acc_each_dset<-0.5)=-0.5;
        
        for i=1:current_many_dsets
            current_delta_acc=delta_acc_each_dset(i);
            cv_results.delta=0;
            idx=randperm(length(sample_sizes));
            current_size=sample_sizes(idx(1));
            dset_sizes(i)=current_size;
            while var(cv_results.delta)==0 && mean(cv_results.delta)==0
                %notice 0.5+abs(current_delta_acc) because we can only simulate positive differences.
                %if the current_delta_acc is negative, we will swap results
                %later.
                [cv_results]=cv(n_runs,nfolds,current_size,0.5+abs(current_delta_acc));
            end
            
            diff_a_b=cv_results.delta;
            if current_delta_acc<0
                diff_a_b=-diff_a_b;
            end
            
            diff_acc_a_b_each_dset(i)=mean(diff_a_b);
            
            %equivalent to p-value one-sided right tailed
            ttest_prob_h1(i)=ttest_Bayesian(diff_a_b,rho,0);
            Diff_ab(:,i)=diff_a_b;
            
            
            
        end
     
        [pdf,cdf]=poisson_bin(ttest_prob_h1);
        
        %the first element of pdf is
        %probability of 0 successes; the last element is the probability of
        %success in every data set. We integrate the pdf of the poission-bin
        %over the following domain: the number of dsets > n/2, obtaining
        %the overall probability of the second algorithm being better than
        %the first.
        ttest_poisson_h1(k)=1-cdf(floor(current_many_dsets/2)+1);       
        
        %frequentist Wilcoxon signed rank
        sign_rank_p_value(k) = signrank(diff_acc_a_b_each_dset,0,'tail','right');
        [~,student_p_value(k)]=ttest(diff_acc_a_b_each_dset,0,'tail','right');
    end
    
    results(j).how_many_dsets=current_many_dsets;
    results(j).delta_acc=delta_acc;
    results(j).ttest_poisson_h1=ttest_poisson_h1;
    results(j).sign_rank_p_value=sign_rank_p_value;
    results(j).student_p_value=student_p_value;
    
end
%save the csv file
tmp_str=strcat(num2str(n_runs),'runs','_theta',num2str(delta_acc(1)),...
    '_dsets',num2str(how_many_dsets(1)));
filename=strcat('multiple_dsets_cauchy_',tmp_str,'.csv');


%save the mat file
save_results(results,filename);
mat_filename=strcat('multiple_dsets_cauchy_',tmp_str,'.mat');


save (mat_filename, 'results');

    function save_results(results,filename)
        fid = fopen(filename,'w');
        fprintf(fid,'%s  \n','mean_delta_acc,cv_runs,num_experiments,how_many_dsets,h1_poisson,h1_sign_rank');
        for ii=1:length(results)
            num_experiments=length(results(ii).ttest_poisson_h1);
            h1_poisson=mean(results(ii).ttest_poisson_h1>.95);
            h1_sign_rank=mean(results(ii).sign_rank_p_value<.05);
            tmp_vector=[h1_poisson;h1_sign_rank];
            
            fprintf(fid,'%1.2f %s %d  %s %d %s %d %s %1.2f %s',results(ii).delta_acc,',',n_runs,',',num_experiments,',',results(ii).how_many_dsets,',');
            for jj=1:length(tmp_vector)-1
                fprintf(fid, '%1.3f %s', tmp_vector(jj),',');
            end
            fprintf(fid, '%1.3f', tmp_vector(end));
            fprintf(fid, '%s', ',');
        end
        fclose(fid);
    end

end
